/**
 * Copyright (C) 2007-2011 Lawrence Murray
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * @author Lawrence Murray <lawrence@indii.org>
 * $Rev: 309 $
 * $Date: 2011-10-20 22:15:34 +0800 (Thu, 20 Oct 2011) $
 */
#include "Help.hpp"

#include "wx/stdpaths.h"
#include "wx/msgdlg.h"
#include "wx/fs_arc.h"
#include "wx/fs_zip.h"

#include <vector>

using namespace indii;

wxHtmlHelpController* Help::help;
bool Help::haveHelp;

#ifdef __WXMSW__
#define HELP_FLAGS (wxHF_DEFAULT_STYLE&~wxHF_BOOKMARKS&~wxHF_INDEX|wxHF_OPEN_FILES)
// ^ wxHF_MERGE_BOOKS starts with contents expanded, but causing errors on Windows
#elif defined(__WXOSX__)
#define HELP_FLAGS (wxHF_CONTENTS|wxHF_SEARCH|wxHF_PRINT|wxHF_MERGE_BOOKS)
#else
#define HELP_FLAGS (wxHF_DEFAULT_STYLE&~wxHF_BOOKMARKS&~wxHF_INDEX|wxHF_OPEN_FILES|wxHF_MERGE_BOOKS)
#endif

void Help::init(wxWindow* parent) {
  wxFileSystem::AddHandler(new wxArchiveFSHandler()); // for *.htb file

  help = new wxHtmlHelpController(HELP_FLAGS, parent);

  haveHelp = false;
  std::vector<wxFileName> files;
  files.push_back(wxFileName(_("tintii.htb")));
  files.push_back(wxFileName(wxStandardPaths::Get().GetResourcesDir(), _("tintii.htb")));
  files.push_back(wxFileName(wxStandardPaths::Get().GetResourcesDir() << wxFileName::GetPathSeparator() << _("Plug-Ins"), _("tintii.htb")));
  #ifdef __WXMSW__
  /* support shortcuts to these files also */
  files.push_back(wxFileName(wxStandardPaths::Get().GetResourcesDir(), _("tintii.htb.lnk")));
  files.push_back(wxFileName(wxStandardPaths::Get().GetResourcesDir() << wxFileName::GetPathSeparator() << _("Plug-Ins"), _("tintii.htb.lnk")));
  #endif
  #ifdef __WXOSX__
  /* for plugin */
  files.push_back(wxFileName(wxStandardPaths::Get().GetResourcesDir() << _("/../../../Plug-ins/tintii.plugin/Contents/Resources"), _("tintii.htb")));
  files.push_back(wxFileName(_("/Library/indii.org/plugins/tintii.htb")));
  #endif
  #ifdef __WXGTK__
  /* datadir from autoconf */
  files.push_back(wxFileName(_(DATADIR)));
  #endif
  std::vector<wxFileName>::iterator iter;
  for (iter = files.begin(); iter != files.end() && !haveHelp; ++iter) {
    iter->Normalize(); // follow shortcuts etc
    if (iter->IsOk() && iter->FileExists()) {
      haveHelp = help->AddBook(*iter);
    }
  }
}

void Help::term() {
  help->Quit();
  delete help;
}

void Help::show(const int section) {
  if (haveHelp) {
    help->Display(section);
    //help->GetFrame()->SetFocus();
		#if defined(__WXOSX__) && defined(__PLUGIN__)
    /* in Photoshop CS5, menus and toolbars obscure help window, and so can't
     * close it, make sure it won't be obscured */
    int x, y;
    help->GetFrame()->GetScreenPosition(&x, &y);
    if (x < 40 || y < 120) {
      x = std::max(40, x);
      y = std::max(120, y);
      help->GetFrame()->Move(x, y);
    }
		#endif
  } else {
    wxMessageBox(wxString("The tintii online help system could not be loaded.\n\nMake sure "
        #ifndef WX_28
        "that the tintii.htb file is in the ") <<
        #else
        "that the tintii.htb file is in the ", wxConvUTF8) <<
        #endif
        wxStandardPaths::Get().GetResourcesDir() <<
        _(" directory, and restart tintii."),
        _("Error"), wxOK|wxICON_HAND, NULL);
  }
}
