/**
 * Copyright (C) 2007-2011 Lawrence Murray
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * @author Lawrence Murray <lawrence@indii.org>
 * $Rev: 308 $
 * $Date: 2011-10-11 21:56:35 +0800 (Tue, 11 Oct 2011) $
 */
#ifndef INDII_GUI_PREVIEWIMAGE_HPP
#define INDII_GUI_PREVIEWIMAGE_HPP

#include "../image/ImageManipulation.hpp"
#include "../image/ImageResource.hpp"
#include "../model/Model.hpp"
#include "../model/Observer.hpp"

#include "wx/scrolwin.h"
#include "wx/bitmap.h"
#include "wx/dcclient.h"

namespace indii {
/**
 * Image preview.
 */
class PreviewImage : public wxScrolledWindow, public Observer {
public:
  /**
   * Constructor.
   *
   * @param parent Parent window.
   * @param res Image resource manager. Caller retains ownership.
   * @param model Cluster model. Caller retains ownership.
   */
  PreviewImage(wxWindow* parent, ImageResource* res, Model* model);

  /**
   * Destructor.
   */
  ~PreviewImage();

  /**
   * Zoom in.
   */
  void zoomIn();

  /**
   * Zoom out.
   */
  void zoomOut();

  /**
   * Zoom to 1:1
   */
  void zoomNormal();

  /**
   * Zoom to fit.
   */
  void zoomFit();

  /**
   * Refresh view.
   */
  void refresh();

  /*
   * Model events.
   */
  virtual void notify();
    
  /**
   * Handle paint event.
   */
  void OnPaint(wxPaintEvent& evt);

  /**
   * Handle resize event.
   */
  void OnSize(wxSizeEvent& evt);

  /*
   * Mouse events.
   */ 
  void OnMouseDown(wxMouseEvent& evt);
  void OnMouseUp(wxMouseEvent& evt);
  void OnMouseWheel(wxMouseEvent& evt);
  void OnMouseMove(wxMouseEvent& evt);
  void OnMouseEnter(wxMouseEvent& evt);
  void OnMouseLeave(wxMouseEvent& evt);
  void OnEraseBackground(wxEraseEvent& evt);

  /**
   * Context sensitive help.
   */
  void OnHelp(wxHelpEvent& evt);

private:
  /**
   * Image resource manager.
   */
  ImageResource* res;

  /**
   * Model.
   */
  Model* model;

  /**
   * Zoom numerator.
   */
  unsigned int zoomNumerator;

  /**
   * Zoom denominator.
   */
  unsigned int zoomDenominator;

  /**
   * Device context should be cleared before next paint (used to control
   * flicker, only clear device context when not doing so may be a problem).
   */
  bool clear;

  /**
   * B&W layer.
   */
  wxBitmap bg;

  /**
   * Alpha channel workspace.
   */
  channel a;

  /*
   * Dragging variables.
   */
  bool dragging;
  int mouseX, mouseY;

  /**
   * Update background.
   *
   * @param rect Rectangular update region.
   */
  void updateBackground();

  DECLARE_EVENT_TABLE()

};
}

#endif
