/**
 * Copyright (C) 2007-2011 Lawrence Murray
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * @author Lawrence Murray <lawrence@indii.org>
 * $Rev: 309 $
 * $Date: 2011-10-20 22:15:34 +0800 (Thu, 20 Oct 2011) $
 */
#include "Registration.hpp"

#include "../reg/Components.hpp"
#include "../../gnome/gnome.hpp"

#include "wx/gbsizer.h"
#include "wx/stattext.h"
#include "wx/textctrl.h"
#include "wx/utils.h"
#include "wx/statbmp.h"

using namespace indii;

#ifdef __WXMAC__
/* modal dialogs shouldn't have close button */
#define TINTREGISTRATION_STYLE wxCAPTION
#else
#define TINTREGISTRATION_STYLE wxDEFAULT_DIALOG_STYLE
#endif

TintRegistration::TintRegistration(wxWindow* parent, const Components::Component c) :
    wxDialog(parent, wxID_ANY, _("Register"), wxDefaultPosition, wxDefaultSize, TINTREGISTRATION_STYLE),
    c(c) {
  /* pre-condition */
  assert (c == Components::APP || c == Components::PLUGIN);
  
  static const int PADDING = 8;

  wxStaticBitmap* bmpKey = new wxStaticBitmap(this, wxID_ANY, *_img_gnome_emblem_web);
  wxStaticText *lblTitle, *lblDesc;
  if (c == Components::APP) {
    if (Components::isActivated(c)) {
	  	lblTitle = new wxStaticText(this, wxID_ANY, _("Your tintii app is already registered"));
      lblDesc = new wxStaticText(this, wxID_ANY, _(""));      
    } else if (Components::isTrial(c)) {
	  	lblTitle = new wxStaticText(this, wxID_ANY, _("Your tintii app is not yet registered"));
      lblDesc = new wxStaticText(this, wxID_ANY, _("Buy a registration code at the URL below and enter it here to register."));
    } else {
      lblTitle = new wxStaticText(this, wxID_ANY, _("The trial period for your tintii app has expired"));
      lblDesc = new wxStaticText(this, wxID_ANY, _("Buy a registration code at the URL below and enter it here to continue using tintii."));
    }
  } else {
  	if (Components::isActivated(Components::PLUGIN)) {
	  	lblTitle = new wxStaticText(this, wxID_ANY, _("Your tintii plugin is already registered"));
      lblDesc = new wxStaticText(this, wxID_ANY, _(""));      
  	} else if (Components::isActivated(Components::APP)) {
      lblTitle = new wxStaticText(this, wxID_ANY, _("Your tintii plugin is not yet registered"));
      lblDesc = new wxStaticText(this, wxID_ANY, _("While you have registered your tintii app, the registration code that you purchased is not valid for the tintii plugin. Buy a registration code for the plugin at the URL below, and enter it here to use the tintii plugin."));
    } else {
      lblTitle = new wxStaticText(this, wxID_ANY, _("Your tintii plugin is not yet registered"));      
      lblDesc = new wxStaticText(this, wxID_ANY, _("Buy a registration code at the URL below and enter it here to use the tintii plugin."));
    }
  }
  wxButton* butUrl = new wxButton(this, ID_BUT_URL,
      _("www.indii.org/software/tintii/register"));

  lblDesc->Wrap(lblTitle->GetSize().GetWidth());
  
  #ifndef __WXMAC__
  wxFont fntUrl(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
  fntUrl.SetUnderlined(true);
  butUrl->SetForegroundColour(*wxBLUE);
  #endif
  #ifndef WX_28
  lblTitle->SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT).Bold());
  #else
  wxFont fntTitle(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
  fntTitle.SetWeight(wxFONTWEIGHT_BOLD);
  lblTitle->SetFont(fntTitle);
  #endif
  lblDesc->SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
        
  txtKey = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition,
      wxSize(160,-1));

  wxGridBagSizer* keySizer = new wxGridBagSizer();
  keySizer->Add(bmpKey, wxGBPosition(0,0), wxDefaultSpan,
      wxALL|wxALIGN_CENTRE_VERTICAL, PADDING);
  keySizer->Add(txtKey, wxGBPosition(0,1), wxDefaultSpan,
      wxALL|wxALIGN_CENTRE_VERTICAL, PADDING);

  wxGridBagSizer* sizer = new wxGridBagSizer();
  sizer->Add(lblTitle, wxGBPosition(0,0), wxDefaultSpan, wxALL|wxEXPAND, PADDING);
  sizer->Add(lblDesc, wxGBPosition(1,0), wxDefaultSpan, wxALL|wxEXPAND, PADDING);
  sizer->Add(keySizer, wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTRE_HORIZONTAL);
  sizer->Add(butUrl, wxGBPosition(3,0), wxDefaultSpan, wxALL|wxALIGN_CENTRE_HORIZONTAL|wxEXPAND, PADDING);

  wxSizer* buttonSizer = this->CreateButtonSizer(wxOK|wxCANCEL);
  sizer->Add(buttonSizer, wxGBPosition(4,0), wxGBSpan(1,2), wxALL|wxEXPAND,
      PADDING);

  SetSizerAndFit(sizer);
}

TintRegistration::~TintRegistration() {  
  //
}

void TintRegistration::OnButtonUrl(wxCommandEvent& evt) {
  wxLaunchDefaultBrowser(_("http://www.indii.org/software/tintii/register"));
}

void TintRegistration::OnButtonOk(wxCommandEvent& evt) {
  if (Components::activate(c, txtKey->GetValue())) {
    EndModal(wxID_OK);
  }
}

void TintRegistration::OnButtonCancel(wxCommandEvent& evt) {
  EndModal(wxID_CANCEL);
}

void TintRegistration::OnClose(wxCloseEvent& evt) {
  Destroy();
}

BEGIN_EVENT_TABLE(TintRegistration, wxDialog)
EVT_CLOSE(TintRegistration::OnClose)
EVT_BUTTON(ID_BUT_URL, TintRegistration::OnButtonUrl)
EVT_BUTTON(wxID_OK, TintRegistration::OnButtonOk)
EVT_BUTTON(wxID_CANCEL, TintRegistration::OnButtonCancel)
END_EVENT_TABLE()
