/**
 * Copyright (C) 2007-2011 Lawrence Murray
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * @author Lawrence Murray <lawrence@indii.org>
 * $Rev: 241 $
 * $Date: 2011-02-20 21:01:45 +0800 (Sun, 20 Feb 2011) $
 */
#ifndef INDII_MODEL_MODEL_HPP
#define INDII_MODEL_MODEL_HPP

#include "Observable.hpp"

#include "wx/image.h"

namespace indii {
/**
 * Abstract model.
 */
class Model : public Observable {
public:
  /**
   * Calculate foreground of subimage.
   *
   * @param rect Rectangular region of interest, relative to scaled image.
   * @param width Scaled width of image.
   * @param height Scaled height of image.
   * @param[out] o Output image.
   */
  virtual void calcFg(const wxRect& rect, const unsigned width,
      const unsigned height, wxImage& o) = 0;

  /**
   * Calculate foreground of whole image.
   *
   * @param width Scaled width of image.
   * @param height Scaled height of image.
   * @param[out] o Output image.
   */
  virtual void calcFg(const unsigned width, const unsigned height,
      wxImage& o) = 0;

};
}

#endif
