/**
 * Copyright (C) 2007-2011 Lawrence Murray
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * @author Lawrence Murray <lawrence@indii.org>
 * $Rev: 308 $
 * $Date: 2011-10-11 21:56:35 +0800 (Tue, 11 Oct 2011) $
 */
#ifndef INDII_MODEL_OBSERVABLE_HPP
#define INDII_MODEL_OBSERVABLE_HPP

#include <set>

namespace indii {
  class Observer;

/**
 * Abstract observable object.
 */
class Observable {
public:
  /**
   * Destructor.
   */
  virtual ~Observable();

  /**
   * Registry type.
   */
  typedef std::set<Observer*> registry_t;

  /**
   * Register observer.
   */
  void subscribe(Observer* o);

  /**
   * Deregister observer.
   */
  void unsubscribe(Observer* o);

  /**
   * Notify all registered observers of change.
   */
  void notifyAll();
  
  /**
   * Notify all registered observers of destruction.
   */
  void notifyDestroy();
   
protected:
  /**
   * Registry of observers.
   */
  registry_t os;
};
}

#endif

