/**
 * Copyright (C) 2007-2011 Lawrence Murray
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * @author Lawrence Murray <lawrence@indii.org>
 * $Rev: 308 $
 * $Date: 2011-10-11 21:56:35 +0800 (Tue, 11 Oct 2011) $
 */
#ifndef INDII_TINT_GUI_CHANNELMIXERPANE_HPP
#define INDII_TINT_GUI_CHANNELMIXERPANE_HPP

#include "TintPane.hpp"
#include "../model/ClusterModel.hpp"
#include "../model/ClusterModelObserver.hpp"

#include "wx/slider.h"

namespace indii {
/**
 * Greyscale channel mixing pane.
 */
class ChannelMixerPane : public TintPane, public ClusterModelObserver {
public:
  /**
   * Constructor.
   *
   * @param parent Parent.
   */
  ChannelMixerPane(wxWindow* parent);

  /**
   * Set model.
   *
   * @param model Model.
   */
  void setModel(ClusterModel* model);  

  /*
   * Handle model changes.
   */
  virtual void notifyGreyscaleChange();

  /*
   * Slider control events.
   */
  void OnSlide(wxScrollEvent& evt);

  /**
   * Context sensitive help.
   */
  void OnHelp(wxHelpEvent& evt);
  
private:
  /**
   * Cluster model.
   */
  ClusterModel* model;

  /*
   * Slider controls.
   */ 
  wxSlider *slideRed, *slideGreen, *slideBlue;

  /*
   * Slider control ids.
   */
  enum ID {
    ID_SLIDE_RED,
    ID_SLIDE_GREEN,
    ID_SLIDE_BLUE
  };

  DECLARE_EVENT_TABLE()

};
}

#endif

