/**
 * Copyright (C) 2007-2011 Lawrence Murray
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * @author Lawrence Murray <lawrence@indii.org>
 * $Rev: 308 $
 * $Date: 2011-10-11 21:56:35 +0800 (Tue, 11 Oct 2011) $
 */
#ifndef INDII_TINT_GUI_PREVIEWPANE_HPP
#define INDII_TINT_GUI_PREVIEWPANE_HPP

#include "TintPane.hpp"
#include "../model/ClusterModel.hpp"
#include "../../gui/PreviewImage.hpp"
#include "../../image/ImageResource.hpp"

#include "wx/panel.h"

namespace indii {
/**
 * Preview pane.
 */
class PreviewPane : public wxPanel {
public:
  /**
   * Constructor.
   *
   * @param parent Parent.
   * @param res Image resource.
   * @param model Cluster model.
   */
  PreviewPane(wxWindow* parent, ImageResource* res, ClusterModel* model);

  /**
   * Get preview image.
   */
  PreviewImage* getPreviewImage();

  /**
   * Context sensitive help.
   */
  void OnHelp(wxHelpEvent& evt);
  
private:
  /**
   * Image resources.
   */
  ImageResource* res;

  /**
   * Cluster model.
   */
  ClusterModel* model;

  /**
   * Preview image.
   */
  PreviewImage* preview;

  DECLARE_EVENT_TABLE()

};
}

inline indii::PreviewImage*
    indii::PreviewPane::getPreviewImage() {
  return preview;
}

#endif

