/**
 * Copyright (C) 2007-2011 Lawrence Murray
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * @author Lawrence Murray <lawrence@indii.org>
 * $Rev: 308 $
 * $Date: 2011-10-11 21:56:35 +0800 (Tue, 11 Oct 2011) $
 */
#include "TintApp.hpp"

#include "../images/tint.hpp"
#include "../../gui/Splash.hpp"
#include "../../gui/About.hpp"
#include "../../gui/Help.hpp"
#include "../../gui/OpenRecentMenu.hpp"
#include "../../reg/Components.hpp"
#include "../../image/FileImageResource.hpp"
#include "../../../gnome/gnome.hpp"

#include "wx/splash.h"
#include "wx/filesys.h"
#include "wx/fs_arc.h"
#include "wx/stdpaths.h"

#ifdef __WXMSW__
#include "omp.h"
#endif

using namespace indii;

#if defined(__WXMSW__) && wxUSE_UNICODE
IMPLEMENT_APP_NO_MAIN(TintApp)

extern "C" int WINAPI wWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
    wchar_t * WXUNUSED(lpCmdLine), int nCmdShow) {
   wxDISABLE_DEBUG_SUPPORT();
   return wxEntry(hInstance, hPrevInstance, NULL, nCmdShow);
}
#else
IMPLEMENT_APP(TintApp)
#endif

void TintApp::MacOpenFile(const wxString &file) {
  wxBeginBusyCursor();
  window = new TintWindow(NULL, file, TintWindow::STANDALONE, this);
  wxEndBusyCursor();
  SetTopWindow(window);
  window->CentreOnScreen();
  window->Show();
  window->Raise();
}

bool TintApp::OnInit() {
  #if defined(__WXMAC__) && defined(MAC_APP_STORE)
	if (!Components::isActivated(Components::APP)) {
    exit(173);
    return false;
  } else {    
    return wxApp::OnInit(); // for command line parsing
  }
  #else
  return wxApp::OnInit();
  #endif
}

void TintApp::OnInitCmdLine(wxCmdLineParser& parser) {
  wxApp::OnInitCmdLine(parser);

  parser.AddOption(_("path"), wxEmptyString, _("Path of image to open"));
}

bool TintApp::OnCmdLineParsed(wxCmdLineParser& parser) {
  if (wxApp::OnCmdLineParsed(parser)) {
    tint_initialize_images();
    wxSplashScreen* splash = new wxSplashScreen(Splash(_img_tint_splash),
        wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_TIMEOUT, 6000, NULL, -1);
    wxInitAllImageHandlers();
    gnome_initialize_images();
    Help::init();
    
    #ifdef __WXOSX__
    wxApp::SetExitOnFrameDelete(false);
    //wxApp::SetMacPreferencesMenuItemId(wxID_NONE);
    wxApp::s_macPreferencesMenuItemId = wxID_NONE;

    /* permanent menu for Mac */
		wxMenuBar* mbMenu = new wxMenuBar();
    mnRecent = new OpenRecentMenu();
    
    wxMenu* mnFile = new wxMenu();
    mnFile->Append(wxID_OPEN, _("&Open...\tCtrl-O"));
    mnFile->AppendSubMenu(mnRecent, _("Open Recent"));
    mnFile->Append(wxID_EXIT); // moved to program menu
      
    wxMenu* mnHelp = new wxMenu();
    mnHelp->Append(wxID_HELP, _("&Help\tCtrl-?"));
    mnHelp->Append(wxID_ABOUT);
	  if (!Components::isActivated(Components::APP)) {
  	  mnHelp->Append(ID_BUT_REGISTER, _("Register"));
    }
    
    mbMenu->Append(mnFile, _("&File"));
    mbMenu->Append(mnHelp, _("&Help"));

    wxMenuBar::MacSetCommonMenuBar(mbMenu);
    #endif
    
    wxString file;
    if (parser.Found(_("path"), &file)) {
      window = new TintWindow(NULL, file, TintWindow::STANDALONE, this);
    } else {
      window = new TintWindow(NULL, TintWindow::STANDALONE, this);
    }
    SetTopWindow(window);
    window->CentreOnScreen();
    window->Show();
    window->Raise();
  
    return true;
  } else {
    return false;
  }
}

int TintApp::OnRun() {
	if (!Components::isActivated(Components::APP) && !Components::isTrial(Components::APP)) {
	  TintRegistration* dlgRegister = new TintRegistration(window, Components::APP);
    dlgRegister->Centre();
    if (dlgRegister->ShowModal() != wxID_OK) {
    	dlgRegister->Destroy();
      window->Close(true);
			window = NULL;
      return false;
    } else {
      dlgRegister->Destroy();
      return wxApp::OnRun();
    }
  } else {
	  return wxApp::OnRun();
  }
}

int TintApp::OnExit() {
  Help::term();
  
  return wxApp::OnExit();
}

void TintApp::OnButtonOpen(wxCommandEvent& evt) {
	if (window != NULL) {
    window->OnButtonOpen(evt);
  } else {
    wxString openPath;
    wxConfig config(_("tintii"), _("indii.org"));
    if (!config.Read(_("OpenPath"), &openPath)) {
      openPath = wxStandardPaths::Get().GetDocumentsDir();
    }
    
    wxString path = wxFileSelector(wxFileSelectorPromptStr, openPath, wxEmptyString,
        wxEmptyString, _("Image Files ") + wxImage::GetImageExtWildcard() +
        _("|All files (*)|*"), wxFD_OPEN|wxFD_CHANGE_DIR|wxFD_FILE_MUST_EXIST|
        wxFD_PREVIEW, NULL);
  	if (!path.empty()) {
      if (wxImage::CanRead(path)) {
        wxFileName filename(path);
	      if (filename.IsOk() && filename.FileExists()) {
  	  		wxBeginBusyCursor();
   		  	window = new TintWindow(NULL, path, TintWindow::STANDALONE, this);
   	  		wxEndBusyCursor();
   	  		SetTopWindow(window);
  	 	  	window->CentreOnScreen();
    	  	window->Show();
   	  		window->Raise();
      
          openPath = filename.GetPath();
        	config.Write(_("OpenPath"), openPath);
          config.Flush();
          
      		OpenRecentMenu::AddFile(path);
        } else {
        	wxMessageBox(_("The selected file does not exist. Please try another."),
          	           _("File does not exist"), wxOK|wxICON_ERROR, NULL);
      	  OpenRecentMenu::RemoveFile(path);
    	  }
      } else {
        wxMessageBox(_("The selected file is of an unsupported type. Please try another."),
                    _("Unsupported type"), wxOK|wxICON_ERROR, NULL);
      }
    }
  }
}

void TintApp::OnOpenRecent(wxCommandEvent& evt) {
	if (window != NULL) {
    window->OnOpenRecent(evt);
  } else {
		int id = evt.GetId();
  	if (id >= 300 && id < 310) {
			wxString path(OpenRecentMenu::GetFile(id));
      if (wxImage::CanRead(path)) {
  			wxFileName filename(path);
    		if (!path.empty() && filename.IsOk() && filename.FileExists()) {
      		wxBeginBusyCursor();
  	    	window = new TintWindow(NULL, path, TintWindow::STANDALONE, this);
   		  	wxEndBusyCursor();
      		SetTopWindow(window);
        	window->CentreOnScreen();
  	    	window->Show();
    	  	window->Raise();
            
      		OpenRecentMenu::AddFile(path);
    		} else {
      		wxMessageBox(_("The selected file does not exist. Please try another."),
        		           _("File does not exist"), wxOK|wxICON_ERROR, NULL);
      		OpenRecentMenu::RemoveFile(path);
    		}
      } else {
        wxMessageBox(_("The selected file is of an unsupported type. Please try another."),
                     _("Unsupported type"), wxOK|wxICON_ERROR, NULL);
        OpenRecentMenu::RemoveFile(path);
      }
    }
  }
}

void TintApp::OnButtonAbout(wxCommandEvent& evt) {
  showAboutDialog();
}

void TintApp::OnButtonRegister(wxCommandEvent& evt) {
	if (window == NULL) {
    TintRegistration* dlgRegister = new TintRegistration(NULL, Components::APP);
	  dlgRegister->Centre();
  	dlgRegister->ShowModal();
    dlgRegister->Destroy();
  } else {
    window->OnButtonRegister(evt);
  }
}

void TintApp::OnButtonHelp(wxCommandEvent& evt) {
  Help::show();
}

void TintApp::OnButtonExit(wxCommandEvent& evt) {
  if (window != NULL) {
    window->OnButtonExit(evt);
  }

  #ifdef __WXOSX__
  if (window == NULL) {
    ExitMainLoop();
  }
  #else
  // handled in notifyClose() instead
  #endif
}

void TintApp::notifyClose(TintWindow* window) {
	assert (this->window == window);

  this->window = NULL;
}

BEGIN_EVENT_TABLE(TintApp, wxApp)
#ifdef __WXOSX__
EVT_MENU(wxID_OPEN, TintApp::OnButtonOpen)
EVT_MENU(wxID_ABOUT, TintApp::OnButtonAbout)
EVT_MENU(wxID_HELP, TintApp::OnButtonHelp)
EVT_MENU(ID_BUT_REGISTER, TintApp::OnButtonRegister)
EVT_MENU(wxID_EXIT, TintApp::OnButtonExit)

EVT_MENU(300, TintApp::OnOpenRecent)
EVT_MENU(301, TintApp::OnOpenRecent)
EVT_MENU(302, TintApp::OnOpenRecent)
EVT_MENU(303, TintApp::OnOpenRecent)
EVT_MENU(304, TintApp::OnOpenRecent)
EVT_MENU(305, TintApp::OnOpenRecent)
EVT_MENU(306, TintApp::OnOpenRecent)
EVT_MENU(307, TintApp::OnOpenRecent)
#endif
END_EVENT_TABLE()
