/**
 * Copyright (C) 2007-2011 Lawrence Murray
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * @author Lawrence Murray <lawrence@indii.org>
 * $Rev: 308 $
 * $Date: 2011-10-11 21:56:35 +0800 (Tue, 11 Oct 2011) $
 */
#ifndef INDII_TINT_GUI_TINTAPP_HPP
#define INDII_TINT_GUI_TINTAPP_HPP

#include "TintWindow.hpp"
#include "TintWindowObserver.hpp"
#include "../model/ClusterModel.hpp"
#include "../../image/ImageResource.hpp"
#include "../../gui/Registration.hpp"

#include "wx/app.h"
#include "wx/cmdline.h"

#include <set>

namespace indii {
/**
 * Tint wxWidgets application.
 */
class TintApp : public wxApp, public TintWindowObserver {
public:
  virtual void MacOpenFile(const wxString &file);
  
  virtual bool OnInit();
  
  void OnInitCmdLine(wxCmdLineParser& parser);

  bool OnCmdLineParsed(wxCmdLineParser& parser);

  virtual int OnRun();

  virtual int OnExit();
 
  /**
   * Open menu item event.
   */
  void OnButtonOpen(wxCommandEvent& evt);

  /**
   * Open recent menu event.
   */
  void OnOpenRecent(wxCommandEvent& evt);
  
  /**
   * About menu item event.
   */
  void OnButtonAbout(wxCommandEvent& evt);
  
  /**
   * Register menu item event.
   */
  void OnButtonRegister(wxCommandEvent& evt);
  
  /**
   * Help menu item event.
   */
  void OnButtonHelp(wxCommandEvent& evt);
  
  /**
   * Exit button event.
   */
  void OnButtonExit(wxCommandEvent& evt);
  
  /**
   * Notify that window is closing.
   *
   * @param window The window being closed.
   */
  virtual void notifyClose(TintWindow* window);
  
private:
  /**
   * Main window.
   */
  TintWindow* window;
  
  /**
   * Open recent menu.
   */
  OpenRecentMenu* mnRecent;
  
  /*
   * Control ids.
   */
  enum ID {
    ID_BUT_DEFAULT = 1, // can't have menu item id of 0 on Mac
    ID_BUT_REGISTER,
    ID_OPEN_RECENT = 765 // must match OpenRecentMenu
  };
  
  DECLARE_EVENT_TABLE()
};
}

#endif
