/**
 * Copyright (C) 2007-2011 Lawrence Murray
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * @author Lawrence Murray <lawrence@indii.org>
 * $Rev: 308 $
 * $Date: 2011-10-11 21:56:35 +0800 (Tue, 11 Oct 2011) $
 */
#ifndef INDII_TINT_GUI_TINTDOCUMENT_HPP
#define INDII_TINT_GUI_TINTDOCUMENT_HPP

#include "PreviewPane.hpp"
#include "../model/ClusterModel.hpp"
#include "../model/ClusterModelObserver.hpp"
#include "../../image/ImageResource.hpp"

#include "wx/frame.h"
#include "wx/aui/aui.h"

namespace indii {
/**
 * Image document.
 */
class TintDocument : public wxPanel, public ClusterModelObserver {
  friend class TintWindow;
public:
  /**
   * Constructor.
   *
   * @param parent Parent.
   * @param res Image resource.
   * @param model Cluster model.
   */
  TintDocument(wxWindow* parent, ImageResource* res, ClusterModel* model);
  
  /**
   * Constructor.
   *
   * @param parent Parent.
   * @param path Path of image to open.
   */
  TintDocument(wxWindow* parent, const wxString& path);
  
  /**
   * Destructor.
   */
  ~TintDocument();
  
  /**
   * Is image saved?
   */
  bool saved();
  
  /**
   * Save image.
   */
  void save();
  
  /**
   * Save image, prompting for path.
   */
  void saveAs();
  
  /**
   * Does image have path?
   */
  bool hasPath();
  
  /**
   * Get image path.
   */
  const wxString& getPath() const;
  
  /**
   * Set image path.
   */
  void setPath(const wxString& path);
  
  virtual void notifyClusterChange(const unsigned int i);
  
  virtual void notifyNumClustersChange();
  
  virtual void notifyNumRepetitionsChange();
  
  virtual void notifySaturationThresholdChange();
  
  virtual void notifyMaxPixelsChange();
  
  virtual void notifySaturationDecayChange();
  
  virtual void notifySaturationSoftnessChange();
  
  virtual void notifyGreyscaleChange();
  
protected:
  /**
   * Preview pane.
   */
  PreviewPane* previewPane;

  /**
   * Image resource.
   */
  ImageResource* res;

  /**
   * Cluster model.
   */
  ClusterModel* model;

private:
  /**
   * Do we own the model and image resource?
   */
  bool own;
  
  /**
   * Is the image, in its current state, saved?
   */
  bool isSaved;
  
  /**
   * Do we have a path for saving the image?
   */
  bool havePath;
  
  /**
   * Path to saved image.
   */
  wxString path;
    
  /**
   * Initialise document.
   */
  void init();
  
  DECLARE_EVENT_TABLE()
};
}

inline bool indii::TintDocument::hasPath() {
  return havePath;
}

inline bool indii::TintDocument::saved() {
  return isSaved;
}

#endif
