/**
 * Copyright (C) 2007-2011 Lawrence Murray
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * @author Lawrence Murray <lawrence@indii.org>
 * $Rev: 225 $
 * $Date: 2010-08-29 14:34:42 +0800 (Sun, 29 Aug 2010) $
 */
#include "WindowMenu.hpp"

#include "TintWindow.hpp"

#include "wx/config.h"

#include <vector>

using namespace indii;

WindowMenu::WindowMenu(TintWindow* window) : window(window) {
  AppendCheckItem(ID_THUMBS, _("Thumbs pane"));
  AppendCheckItem(ID_PREPROCESSING, _("Preprocessing pane"));
  AppendCheckItem(ID_POSTPROCESSING, _("Postprocessing pane"));
  AppendCheckItem(ID_MIXER, _("Channel mixer pane"));
  AppendSeparator();
  Append(ID_DEFAULTS, _("Restore Default Layout"));
}

void WindowMenu::OnShow(wxMenuEvent& evt) {
  Check(ID_THUMBS, window->isThumbPaneShown());
  Check(ID_PREPROCESSING, window->isPrePaneShown());
  Check(ID_POSTPROCESSING, window->isPostPaneShown());
  Check(ID_MIXER, window->isMixerPaneShown());
}

void WindowMenu::OnPreProcessPane(wxCommandEvent& evt) {
  window->showPrePane(!window->isPrePaneShown());
}

void WindowMenu::OnPostProcessPane(wxCommandEvent& evt) {
  window->showPostPane(!window->isPostPaneShown());
}

void WindowMenu::OnMixerPane(wxCommandEvent& evt) {
  window->showMixerPane(!window->isMixerPaneShown());
}

void WindowMenu::OnThumbsPane(wxCommandEvent& evt) {
  window->showThumbPane(!window->isThumbPaneShown());
}

void WindowMenu::OnRestoreDefaults(wxCommandEvent& evt) {
  window->layoutDefault();
}

BEGIN_EVENT_TABLE(WindowMenu, wxMenu)
#ifdef __WXOSX__
/* events handled here on Mac, in TintWindow on other platforms */
EVT_MENU_OPEN(WindowMenu::OnShow)
EVT_MENU(ID_PREPROCESSING, WindowMenu::OnPreProcessPane)
EVT_MENU(ID_POSTPROCESSING, WindowMenu::OnPostProcessPane)
EVT_MENU(ID_MIXER, WindowMenu::OnMixerPane)
EVT_MENU(ID_THUMBS, WindowMenu::OnThumbsPane)
EVT_MENU(ID_DEFAULTS, WindowMenu::OnRestoreDefaults)
#endif
END_EVENT_TABLE()
